clear all;
disp('====Image Deconvolution Using Data-Driven Non-Local Wavelet Frames====');
im_name = 'cameraman256.png';
%% Set degradation
kernel = fspecial('disk', 3);
A = @(x) imfilter(x, kernel, 'circular');
AT = @(x) imfilter(x, rot90(kernel,2), 'circular');            
noise_sig = 2;

%% Generate test image
im_gt = double(imread(im_name));
im_d = A(im_gt);
randn('state', 0);
im_d = im_d + noise_sig * randn(size(im_d));

%% Set parameters and options
% Parmaters of the algorithm
lambda = [0.05 0.05];
delta = 1;
tau1 = [2*1.2e-3 2*1.2e-3];
tau2 = [2*1e-3 2*1e-3];
nloop_in = [30 30];
nlevel = 1;
W = @(x) swt2(x, 3, nlevel);
WT = @(x) iswt2(x, 3, nlevel); 
% Options for block matching
bm_opts.sz_blk = 5;
bm_opts.step = 1;
bm_opts.radius = 25;
bm_opts.n_blk = 16;
bm_opts.ext_mode = 'symmetric';
% Options for setting the weights of the nonlocal and local regularizations
pa_opts.nsig = noise_sig;

%% Deconvolution and output the results
im_r = LNL_Frame_Deconv(im_d, A, AT, W, WT, tau1, tau2, nloop_in, lambda, delta, bm_opts, pa_opts);
psnr_r = 10*log10(255^2 /mean((im_r(:)-im_gt(:)).^2));
figure; imshow(uint8(im_d)); title('degraded image');
figure; imshow(uint8(im_r)); title('restored image');
display(['restored psnr = ', num2str(psnr_r)]);
